#include <iostream>
#include <thread>
#include <sstream>
#include <chrono>
#include <amqpcpp.h>
#include "../conn_handler.h"

int main() {
    std::stringstream ss;
    ss << std::chrono::duration_cast<std::chrono::microseconds>(
            std::chrono::high_resolution_clock::now().time_since_epoch()).count();
    std::string correlation = ss.str();
    ConnHandler handler;
    // 创建连接
    AMQP::TcpConnection connection(handler,
                                   AMQP::Address("amqp-xxx.rabbitmq.ap-sh.public.tencenttdmq.com", 5672,
                                                 AMQP::Login("admin",
                                                             "eyJrZXlJZC......"),
                                                 "amqp-xxx|vhost-cpp"));
    // 建立通道
    AMQP::TcpChannel channel(&connection);
    channel.onError([&handler](const char *message) {
        std::cout << "Channel error: " << message << std::endl;
        handler.Stop();
    });
    // 声明消息队列
    channel.declareQueue(AMQP::exclusive)
            .onSuccess
                    (
                            [&channel, correlation](const std::string &name, int, int) {
                                std::string body = "30";
                                AMQP::Envelope env(body.data(), sizeof(body.data()));
                                env.setCorrelationID(correlation);
                                env.setReplyTo(name);
                                channel.publish("", "rpc_queue", env);
                                std::cout << "Requesting fib(30)" << std::endl;
                            }
                    );
    // 消费消息
    channel.consume("", AMQP::noack)
            .onReceived
                    (
                            [correlation, &handler](const AMQP::Message &m, uint64_t, bool) {
                                if (m.correlationID() != correlation)
                                    return; // just skip it
                                std::cout << "Got " << m.body() << std::endl;
                                handler.Stop();
                            }
                    );
    handler.Start();
    std::cout << "Closing connection." << std::endl;
    connection.close();
    return 0;
}