#include <iostream>
#include <thread>
#include <chrono>
#include <amqpcpp.h>
#include "../conn_handler.h"

// 模拟rpc业务
int fib(int n) {
    switch (n) {
        case 0:
            return 0;
        case 1:
            return 1;
        default:
            return fib(n - 1) + fib(n - 2);
    }
}

int main() {
    ConnHandler handler;
    // 建立连接
    AMQP::TcpConnection connection(handler,
                                   AMQP::Address("amqp-xxx.rabbitmq.ap-sh.public.tencenttdmq.com", 5672,
                                                 AMQP::Login("admin",
                                                             "eyJrZXlJZC......"),
                                                 "amqp-xxx|vhost-cpp"));
    // 建立通道
    AMQP::TcpChannel channel(&connection);
    channel.onError([&handler](const char *message) {
        std::cout << "Channel error: " << message << std::endl;
        handler.Stop();
    });
    channel.setQos(1);
    // 声明消息队列
    channel.declareQueue("rpc_queue");
    // 消费消息
    channel.consume("")
            .onReceived
                    (
                            [&channel](const AMQP::Message &m, uint64_t tag, bool) {
                                std::cout << "fib(" << m.body() << ")" << std::endl;
                                // 处理业务
                                std::string body = std::to_string(fib(std::stoi(m.body())));
                                AMQP::Envelope envelope(body.data(), sizeof(body.data()));
                                envelope.setCorrelationID(m.correlationID());
                                // 回复rpc
                                channel.publish("", m.replyTo(), envelope);
                                channel.ack(tag);
                            }
                    );
    handler.Start();
    std::cout << "Closing connection." << std::endl;
    connection.close();
    return 0;
}
